@@ -107,7 +107,8 @@ private:
 
 RimeEngine::RimeEngine(Instance *instance)
     : instance_(instance), api_(rime_get_api()),
-      factory_([this](InputContext &ic) { return new RimeState(this, ic); }) {
+      factory_([this](InputContext &ic) { return new RimeState(this, ic); }),
+      sessionPool_(this, instance_->globalConfig().shareInputState()) {
     imAction_ = std::make_unique<IMAction>(this);
     instance_->userInterfaceManager().registerAction("fcitx-rime-im",
                                                      imAction_.get());
@@ -137,6 +138,13 @@ RimeEngine::RimeEngine(Instance *instance)
     });
     instance_->userInterfaceManager().registerAction("fcitx-rime-sync",
                                                      &syncAction_);
+    globalConfigReloadHandle_ = instance_->watchEvent(
+        EventType::GlobalConfigReloaded, EventWatcherPhase::Default,
+        [this](Event &) {
+            releaseAllSession();
+            sessionPool_.setPropertyPropagatePolicy(
+                instance_->globalConfig().shareInputState());
+        });
     reloadConfig();
 }
 
@@ -161,6 +169,7 @@ void RimeEngine::rimeStart(bool fullcheck) {
     auto userDir = stringutils::joinPath(
         StandardPath::global().userDirectory(StandardPath::Type::PkgData),
         "rime");
+    RIME_DEBUG() << "Rime data directory: " << userDir;
     if (!fs::makePath(userDir)) {
         if (!fs::isdir(userDir)) {
             RIME_ERROR() << "Failed to create user directory: " << userDir;
@@ -175,6 +184,26 @@ void RimeEngine::rimeStart(bool fullcheck) {
     fcitx_rime_traits.distribution_name = "Rime";
     fcitx_rime_traits.distribution_code_name = "fcitx-rime";
     fcitx_rime_traits.distribution_version = FCITX_RIME_VERSION;
+#ifndef FCITX_RIME_NO_LOG_LEVEL
+    switch (rime().logLevel()) {
+    case NoLog:
+        fcitx_rime_traits.min_log_level = 4;
+        break;
+    case Fatal:
+        fcitx_rime_traits.min_log_level = 3;
+        break;
+    case Error:
+    case Warn:
+    case Info:
+        fcitx_rime_traits.min_log_level = 2;
+        break;
+    case Debug:
+    default:
+        // Rime info is too noisy.
+        fcitx_rime_traits.min_log_level = 0;
+        break;
+    }
+#endif
 
 #ifdef FCITX_RIME_LOAD_PLUGIN
     std::vector<const char *> modules;
@@ -200,10 +229,14 @@ void RimeEngine::rimeStart(bool fullcheck) {
     }
     api_->initialize(&fcitx_rime_traits);
     api_->set_notification_handler(&rimeNotificationHandler, this);
-    if (api_->start_maintenance(fullcheck)) {
-        api_->deploy_config_file("fcitx5.yaml", "config_version");
-    }
+    api_->start_maintenance(fullcheck);
 
+    if (!api_->is_maintenance_mode()) {
+        updateAppOptions();
+    }
+}
+
+void RimeEngine::updateAppOptions() {
     appOptions_.clear();
     RimeConfig config = {NULL};
     if (api_->config_open("fcitx5", &config)) {
@@ -211,6 +244,7 @@ void RimeEngine::rimeStart(bool fullcheck) {
         api_->config_close(&config);
     }
     RIME_DEBUG() << "App options are " << appOptions_;
+    releaseAllSession();
 }
 
 void RimeEngine::reloadConfig() {
@@ -326,7 +360,11 @@ void RimeEngine::reset(const InputMethodEntry &, InputContextEvent &event) {
     inputContext->updateUserInterface(UserInterfaceComponent::InputPanel);
 }
 
-void RimeEngine::save() {}
+void RimeEngine::save() {
+    // Block notification for 5 sec.
+    blockNotificationBefore_ = now(CLOCK_MONOTONIC) + 5000000;
+    sync();
+}
 
 void RimeEngine::rimeNotificationHandler(void *context, RimeSessionId session,
                                          const char *messageType,
@@ -343,6 +381,9 @@ void RimeEngine::rimeNotificationHandler(void *context, RimeSessionId session,
 
 void RimeEngine::notify(const std::string &messageType,
                         const std::string &messageValue) {
+    if (now(CLOCK_MONOTONIC) < blockNotificationBefore_) {
+        return;
+    }
     const char *message = nullptr;
     const char *icon = "";
     const char *tipId = "";
@@ -355,6 +396,11 @@ void RimeEngine::notify(const std::string &messageType,
         } else if (messageValue == "success") {
             message = _("Rime is ready.");
             updateSchemaMenu();
+            if (!api_->is_maintenance_mode()) {
+                api_->deploy_config_file("fcitx5.yaml", "config_version");
+                updateAppOptions();
+                releaseAllSession();
+            }
         } else if (messageValue == "failure") {
             message = _("Rime has encountered an error. "
                         "See /tmp/rime.fcitx.ERROR for details.");
@@ -369,10 +415,10 @@ void RimeEngine::notify(const std::string &messageType,
             message = _("Full Shape is enabled.");
         } else if (messageValue == "!ascii_punct") {
             tipId = "fcitx-rime-ascii-punct";
-            message = _("Full width punctuation is enabled.");
+            message = _("Punctuation conversion is enabled.");
         } else if (messageValue == "ascii_punct") {
             tipId = "fcitx-rime-ascii-punct";
-            message = _("Half width punctuation is enabled.");
+            message = _("Punctuation conversion is disabled.");
         } else if (messageValue == "!simplification") {
             tipId = "fcitx-rime-simplification";
             message = _("Traditional Chinese is enabled.");
@@ -441,19 +487,27 @@ std::string RimeEngine::subModeIconImpl(const InputMethodEntry &,
     return result;
 }
 
-void RimeEngine::deploy() {
-    RIME_DEBUG() << "Rime Deploy";
+void RimeEngine::releaseAllSession() {
     instance_->inputContextManager().foreach([this](InputContext *ic) {
         if (auto state = this->state(ic)) {
             state->release();
         }
         return true;
     });
+}
+
+void RimeEngine::deploy() {
+    RIME_DEBUG() << "Rime Deploy";
+    releaseAllSession();
     api_->finalize();
     rimeStart(true);
 }
 
-void RimeEngine::sync() { api_->sync_user_data(); }
+void RimeEngine::sync() {
+    RIME_DEBUG() << "Rime Sync user data";
+    api_->sync_user_data();
+    releaseAllSession();
+}
 
 void RimeEngine::updateSchemaMenu() {
     if (!api_) {
